@@ -358,10 +358,6 @@ public:
     /// Return the error message that explains why the file is broken.
     string_view broken_error_message() const { return m_broken_message; }
 
-    // For udim only, turn the udim spec filename into a concrete filename
-    // for the given u and v tile indices.
-    std::string udim_to_concrete(int utile, int vtile);
-
     // Return the regex wildcard matching pattern for a udim spec.
     static std::string udim_to_wildcard(string_view udimpattern);
 
@@ -493,7 +489,7 @@ struct TileID {
     /// Initialize a TileID based on full elaboration of image file,
     /// subimage, and tile x,y,z indices.
     TileID(ImageCacheFile& file, int subimage, int miplevel, int x, int y,
-           int z = 0, int chbegin = 0, int chend = -1)
+           int z, int chbegin, int chend)
         : m_x(x)
         , m_y(y)
         , m_z(z)
@@ -503,9 +499,10 @@ struct TileID {
         , m_chend(chend)
         , m_file(&file)
     {
-        int nc = file.spec(subimage, miplevel).nchannels;
-        if (chend < chbegin || chend > nc)
+        if (chend < chbegin) {
+            int nc  = file.spec(subimage, miplevel).nchannels;
             m_chend = nc;
+        }
     }
 
     /// Destructor is trivial, because we don't hold any resources
@@ -938,11 +935,6 @@ public:
                               const ImageSpec* config     = nullptr,
                               bool replace                = false);
 
-    // Return the ImageCacheFile* of the file if it's already in the cache,
-    // otherwise return nullptr and don't open or add the file.
-    ImageCacheFile* find_file_no_add(ustring filename,
-                                     ImageCachePerThreadInfo* thread_info);
-
     /// Verify & prep the ImageCacheFile record for the named image,
     /// return the pointer (which may have changed for deduplication),
     /// or NULL if no such file can be found. This returns a plain old
