@@ -190,8 +190,9 @@ private:
         m_tiled_output_part.reset();
         m_deep_scanline_output_part.reset();
         m_deep_tiled_output_part.reset();
-        m_subimage = -1;
-        m_miplevel = -1;
+        m_levelmode = Imf::ONE_LEVEL;
+        m_subimage  = -1;
+        m_miplevel  = -1;
         m_subimagespecs.clear();
         m_subimagespecs.shrink_to_fit();
         m_headers.clear();
@@ -1346,9 +1347,12 @@ OpenEXROutput::close()
 
 
 bool
-OpenEXROutput::write_scanline(int y, int /*z*/, TypeDesc format,
-                              const void* data, stride_t xstride)
+OpenEXROutput::write_scanline(int y, int z, TypeDesc format, const void* data,
+                              stride_t xstride)
 {
+#if 1
+    return write_scanlines(y, y + 1, z, format, data, xstride, AutoStride);
+#else
     if (!(m_output_scanline || m_scanline_output_part)) {
         errorf("called OpenEXROutput::write_scanline without an open file");
         return false;
@@ -1400,6 +1404,7 @@ OpenEXROutput::write_scanline(int y, int /*z*/, TypeDesc format,
     // FIXME -- can we checkpoint the file?
 
     return true;
+#endif
 }
 
 
