@@ -116,8 +116,8 @@ void declare_texturesystem (py::module& m);
 
 // bool PyProgressCallback(void*, float);
 // object C_array_to_Python_array (const char *data, TypeDesc type, size_t size);
-const char * python_array_code (TypeDesc format);
 TypeDesc typedesc_from_python_array_code (string_view code);
+// const char * python_array_code (TypeDesc format);  // unused
 
 
 inline std::string
