@@ -88,10 +88,15 @@ static bool invalidate_before_iter = true;
 static bool close_before_iter      = false;
 static bool runstats               = false;
 static bool udim_tests             = false;
+static bool do_gettextureinfo      = true;
+static int subimage                = -1;
+static std::string subimagename;
 static Imath::M33f xform;
 static std::string texoptions;
 static std::string gtiname;
 static std::string maketest_template;
+static int maketest_res   = 2048;
+static int maketest_chans = 4;
 static int num_test_files = 0;
 static std::vector<std::string> filenames_to_delete;
 const int pieces_per_udim = 20;
@@ -104,6 +109,9 @@ typedef void (*Mapping2D)(const int&, const int&, float&, float&, float&,
 typedef void (*Mapping3D)(const int&, const int&, Imath::V3f&, Imath::V3f&,
                           Imath::V3f&, Imath::V3f&);
 
+typedef void (*MappingEnv)(const int&, const int&, Imath::V3f&, Imath::V3f&,
+                           Imath::V3f&);
+
 typedef void (*Mapping2DWide)(const Tex::IntWide&, const Tex::IntWide&,
                               Tex::FloatWide&, Tex::FloatWide&, Tex::FloatWide&,
                               Tex::FloatWide&, Tex::FloatWide&,
@@ -114,6 +122,11 @@ typedef void (*Mapping3DWide)(const Tex::IntWide&, const Tex::IntWide&,
                               Imath::Vec3<Tex::FloatWide>&,
                               Imath::Vec3<Tex::FloatWide>&);
 
+typedef void (*MappingEnvWide)(const Tex::IntWide&, const Tex::IntWide&,
+                               Imath::Vec3<Tex::FloatWide>&,
+                               Imath::Vec3<Tex::FloatWide>&,
+                               Imath::Vec3<Tex::FloatWide>&);
+
 
 
 static void
@@ -241,10 +254,20 @@ getargs(int argc, const char* argv[])
       .help("Print runtime statistics");
     ap.arg("--maketests %d:NUMFILES %s:TEMPLATE", &num_test_files,  &maketest_template)
       .help("Make tests from a template (e.g., \"tmp/test{:04}.exr\")");
+    ap.arg("--maketest-res %d:RES", &maketest_res)
+      .help("Resolution for maketests (default: 2048)");
+    ap.arg("--maketest-chans %d:NCHANS", &maketest_chans)
+      .help("Channels for maketests (default: 4)");
     ap.arg("--udim", &udim_tests)
       .help("Do udim-oriented tests");
     ap.arg("--bluenoise", &use_bluenoise)
       .help("Use blue noise for stochastic choices");
+    ap.arg("--no-gettextureinfo %!", &do_gettextureinfo)
+      .help("Do not test gettextureinfo");
+    ap.arg("--subimage %d:INDEX", &subimage)
+      .help("Use the specified subimage (by index)");
+    ap.arg("--subimagename %s:NAME", &subimagename)
+      .help("Use the specified subimage (by name)");
 
     // clang-format on
     ap.parse(argc, argv);
@@ -276,6 +299,10 @@ initialize_opt(TextureOpt& opt)
     opt.anisotropic = anisomax;
     opt.mipmode     = (TextureOpt::MipMode)mipmode;
     opt.interpmode  = (TextureOpt::InterpMode)interpmode;
+    if (subimage >= 0)
+        opt.subimage = subimage;
+    else if (!subimagename.empty())
+        opt.subimagename = ustring(subimagename);
 }
 
 
@@ -301,6 +328,10 @@ initialize_opt(TextureOptBatch& opt)
     opt.anisotropic = anisomax;
     opt.mipmode     = MipMode(mipmode);
     opt.interpmode  = InterpMode(interpmode);
+    if (subimage >= 0)
+        opt.subimage = subimage;
+    else if (!subimagename.empty())
+        opt.subimagename = ustring(subimagename);
 }
 
 
@@ -310,19 +341,28 @@ test_gettextureinfo(ustring filename)
 {
     bool ok;
 
-    int res[2] = { 0 };
-    ok         = texsys->get_texture_info(filename, 0, ustring("resolution"),
+    int res[3] = { 0 };
+
+    ok = texsys->get_texture_info(filename, 0, ustring("resolution"),
                                   TypeDesc(TypeDesc::INT, 2), res);
-    Strutil::print("Result of get_texture_info resolution = {} {}x{}\n", ok,
-                   res[0], res[1]);
+    Strutil::print(
+        "Result of get_texture_info resolution (as int[2]) = {} {}x{}\n", ok,
+        res[0], res[1]);
+    ok = texsys->get_texture_info(filename, 0, ustring("resolution"),
+                                  TypeDesc(TypeDesc::INT, 3), res);
+    Strutil::print(
+        "Result of get_texture_info resolution (as int[3]) = {} {}x{}x{}\n", ok,
+        res[0], res[1], res[2]);
 
     int chan = 0;
-    ok       = texsys->get_texture_info(filename, 0, ustring("channels"),
+
+    ok = texsys->get_texture_info(filename, 0, ustring("channels"),
                                   TypeDesc::INT, &chan);
     Strutil::print("Result of get_texture_info channels = {} {}\n", ok, chan);
 
     float fchan = 0;
-    ok          = texsys->get_texture_info(filename, 0, ustring("channels"),
+
+    ok = texsys->get_texture_info(filename, 0, ustring("channels"),
                                   TypeDesc::FLOAT, &fchan);
     Strutil::print("Result of get_texture_info channels = {} {:g}\n", ok,
                    fchan);
@@ -333,11 +373,28 @@ test_gettextureinfo(ustring filename)
     Strutil::print("Result of get_texture_info data format = {} {}\n", ok,
                    TypeDesc((TypeDesc::BASETYPE)dataformat).c_str());
 
-    const char* datetime = NULL;
-    ok = texsys->get_texture_info(filename, 0, ustring("DateTime"),
-                                  TypeDesc::STRING, &datetime);
-    Strutil::print("Result of get_texture_info datetime = {} {}\n", ok,
-                   (datetime ? datetime : ""));
+    int window[6];
+    ok = texsys->get_texture_info(filename, 0, ustring("datawindow"),
+                                  TypeDesc("int[4]"), window);
+    Strutil::print(
+        "Result of get_texture_info datawindow (as int[4]) = {} [{} {} {} {}]\n",
+        ok, window[0], window[1], window[2], window[3]);
+    ok = texsys->get_texture_info(filename, 0, ustring("datawindow"),
+                                  TypeDesc("int[6]"), window);
+    Strutil::print(
+        "Result of get_texture_info datawindow (as int[6]) = {} [{} {} {} {} {} {}]\n",
+        ok, window[0], window[1], window[2], window[3], window[4], window[5]);
+
+    ok = texsys->get_texture_info(filename, 0, ustring("displaywindow"),
+                                  TypeDesc("int[4]"), window);
+    Strutil::print(
+        "Result of get_texture_info displaywindow (as int[4]) = {} [{} {} {} {}]\n",
+        ok, window[0], window[1], window[2], window[3]);
+    ok = texsys->get_texture_info(filename, 0, ustring("displaywindow"),
+                                  TypeDesc("int[6]"), window);
+    Strutil::print(
+        "Result of get_texture_info displaywindow (as int[6]) = {} [{} {} {} {} {} {}]\n",
+        ok, window[0], window[1], window[2], window[3], window[4], window[5]);
 
     float avg[4];
     ok = texsys->get_texture_info(filename, 0, ustring("averagecolor"),
@@ -345,19 +402,27 @@ test_gettextureinfo(ustring filename)
     Strutil::print("Result of get_texture_info averagecolor = {}",
                    ok ? "yes" : "no\n");
     if (ok)
-        Strutil::print(" {} {} {} {}\n", avg[0], avg[1], avg[2], avg[3]);
+        Strutil::print(" {:g} {:g} {:g} {:g}\n", avg[0], avg[1], avg[2],
+                       avg[3]);
     ok = texsys->get_texture_info(filename, 0, ustring("averagealpha"),
                                   TypeFloat, avg);
     Strutil::print("Result of get_texture_info averagealpha = {}",
                    ok ? "yes" : "no\n");
     if (ok)
-        Strutil::print(" {}\n", avg[0]);
+        Strutil::print(" {:g}\n", avg[0]);
     ok = texsys->get_texture_info(filename, 0, ustring("constantcolor"),
                                   TypeDesc(TypeDesc::FLOAT, 4), avg);
     Strutil::print("Result of get_texture_info constantcolor = {}",
                    ok ? "yes" : "no\n");
     if (ok)
-        Strutil::print(" {} {} {} {}\n", avg[0], avg[1], avg[2], avg[3]);
+        Strutil::print(" {:g} {:g} {:g} {:g}\n", avg[0], avg[1], avg[2],
+                       avg[3]);
+    ok = texsys->get_texture_info(filename, 0, ustring("constantalpha"),
+                                  TypeFloat, avg);
+    Strutil::print("Result of get_texture_info constantalpha = {}",
+                   ok ? "yes" : "no\n");
+    if (ok)
+        Strutil::print(" {:g}\n", avg[0]);
 
     const char* texturetype = NULL;
     ok = texsys->get_texture_info(filename, 0, ustring("textureformat"),
@@ -379,16 +444,6 @@ warp(const Float& x, const Float& y, const Imath::M33f& xform)
 }
 
 
-template<typename Float = float>
-inline Imath::Vec3<Float>
-warp(const Float& x, const Float& y, const Float& z, const Imath::M33f& xform)
-{
-    Imath::Vec3<Float> coord(x, y, z);
-    coord *= xform;
-    return coord;
-}
-
-
 template<typename Float = float>
 inline Imath::Vec2<Float>
 warp_coord(const Float& x, const Float& y)
@@ -550,34 +605,60 @@ map_default_3D(const Int& x, const Int& y, Imath::Vec3<Float>& P,
 
 
 
-template<typename Float = float, typename Int = int>
+// Just map pixels to environment direction
 void
-map_warp_3D(const Int& x, const Int& y, Imath::Vec3<Float>& P,
-            Imath::Vec3<Float>& dPdx, Imath::Vec3<Float>& dPdy,
-            Imath::Vec3<Float>& dPdz)
+map_env_latlong(const int& x, const int& y, Imath::Vec3<float>& R,
+                Imath::Vec3<float>& dRdx, Imath::Vec3<float>& dRdy)
 {
-    Imath::Vec3<Float> coord = warp((Float(x) / output_xres),
-                                    (Float(y) / output_yres), Float(0.5),
-                                    xform);
-    coord.x *= sscale;
-    coord.y *= tscale;
-    coord += texoffset;
-    Imath::Vec3<Float> coordx = warp((Float(x + 1) / output_xres),
-                                     (Float(y) / output_yres), Float(0.5),
-                                     xform);
-    coordx.x *= sscale;
-    coordx.y *= tscale;
-    coordx += texoffset;
-    Imath::Vec3<Float> coordy = warp((Float(x) / output_xres),
-                                     (Float(y + 1) / output_yres), Float(0.5),
-                                     xform);
-    coordy.x *= sscale;
-    coordy.y *= tscale;
-    coordy += texoffset;
-    P    = coord;
-    dPdx = coordx - coord;
-    dPdy = coordy - coord;
-    dPdz.setValue(0, 0, 0);
+    float u         = (x + 0.5f) / output_xres;
+    float v         = (y + 0.5f) / output_yres;
+    float du_dx     = float(1.0f / output_xres);  // du_dy = 0
+    float dv_dy     = float(1.0f / output_yres);  // dv_dx = 0
+    float theta     = float(2.0 * M_PI) * u;
+    float dtheta_dx = float(2.0 * M_PI) * du_dx;  // dtheta_dy = 0
+    float phi       = float(M_PI) * v;
+    float dphi_dy   = float(M_PI) * dv_dy;  // dphi_dx = 0
+
+    R    = Imath::Vec3<float>(fast_sin(phi) * fast_sin(theta), fast_cos(phi),
+                           -fast_sin(phi) * fast_cos(theta));
+    dRdx = Imath::Vec3<float>(fast_sin(phi) * fast_cos(theta) * dtheta_dx,
+                              float(0.0f),
+                              fast_sin(phi) * fast_sin(theta) * dtheta_dx);
+    dRdy = Imath::Vec3<float>(-fast_cos(phi) * fast_sin(theta) * dphi_dy,
+                              -fast_sin(phi) * dphi_dy,
+                              -fast_cos(phi) * fast_cos(theta) * dphi_dy);
+}
+
+
+
+// Turn v3f[lanes] into simd[3]
+inline Imath::Vec3<Tex::FloatWide>
+soa(const Imath::V3f v[])
+{
+    Imath::Vec3<Tex::FloatWide> result;
+    for (int i = 0; i < Tex::BatchWidth; ++i) {
+        result[0][i] = v[i].x;
+        result[1][i] = v[i].y;
+        result[2][i] = v[i].z;
+    }
+    return result;
+}
+
+
+
+// FIXME -- templatize map_env_latlong. For now, just loop over scalar version.
+void
+map_env_latlong(const Tex::IntWide& x, const Tex::IntWide& y,
+                Imath::Vec3<Tex::FloatWide>& R,
+                Imath::Vec3<Tex::FloatWide>& dRdx,
+                Imath::Vec3<Tex::FloatWide>& dRdy)
+{
+    Imath::V3f r[Tex::BatchWidth], drdx[Tex::BatchWidth], drdy[Tex::BatchWidth];
+    for (int i = 0; i < Tex::BatchWidth; ++i)
+        map_env_latlong(x[i], y[i], r[i], drdx[i], drdy[i]);
+    R    = soa(r);
+    dRdx = soa(drdx);
+    dRdy = soa(drdy);
 }
 
 
@@ -883,9 +964,14 @@ tex3d_region(ImageBuf& image, ustring filename, Mapping3D mapping, ROI roi)
         mapping(p.x(), p.y(), P, dPdx, dPdy, dPdz);
 
         // Call the texture system to do the filtering.
-        bool ok = texsys->texture3d(texture_handle, perthread_info, opt, P,
-                                    dPdx, dPdy, dPdz, nchannels, result,
-                                    dresultds, dresultdt, dresultdr);
+        bool ok = use_handle
+                      ? texsys->texture3d(texture_handle, perthread_info, opt,
+                                          P, dPdx, dPdy, dPdz, nchannels,
+                                          result, dresultds, dresultdt,
+                                          dresultdr)
+                      : texsys->texture3d(filename, opt, P, dPdx, dPdy, dPdz,
+                                          nchannels, result, dresultds,
+                                          dresultdt, dresultdr);
         if (!ok) {
             std::string e = texsys->geterror();
             if (!e.empty())
@@ -932,13 +1018,19 @@ tex3d_region_batch(ImageBuf& image, ustring filename, Mapping3DWide mapping,
             RunMask mask = RunMaskOn >> (BatchWidth - npoints);
 
             // Call the texture system to do the filtering.
-            if (y == 0 && x == 0)
-                Strutil::print("P = {}\n", P);
-            bool ok = texsys->texture3d(texture_handle, perthread_info, opt,
-                                        mask, (float*)&P, (float*)&dPdx,
-                                        (float*)&dPdy, (float*)&dPdz, nchannels,
-                                        (float*)result, (float*)dresultds,
-                                        (float*)dresultdt, (float*)dresultdr);
+            bool ok
+                = use_handle
+                      ? texsys->texture3d(texture_handle, perthread_info, opt,
+                                          mask, (float*)&P, (float*)&dPdx,
+                                          (float*)&dPdy, (float*)&dPdz,
+                                          nchannels, (float*)result,
+                                          (float*)dresultds, (float*)dresultdt,
+                                          (float*)dresultdr)
+                      : texsys->texture3d(filename, opt, mask, (float*)&P,
+                                          (float*)&dPdx, (float*)&dPdy,
+                                          (float*)&dPdz, nchannels,
+                                          (float*)result, (float*)dresultds,
+                                          (float*)dresultdt, (float*)dresultdr);
             if (!ok) {
                 std::string e = texsys->geterror();
                 if (!e.empty())
@@ -962,7 +1054,7 @@ tex3d_region_batch(ImageBuf& image, ustring filename, Mapping3DWide mapping,
 void
 test_texture3d(ustring filename, Mapping3D mapping)
 {
-    Strutil::print("Testing 3d texture {}, output = {}\n", filename,
+    Strutil::print("Testing texture3d {}, output = {}\n", filename,
                    output_filename);
     int nchannels = nchannels_override ? nchannels_override : 4;
     ImageSpec outspec(output_xres, output_yres, nchannels, TypeDesc::FLOAT);
@@ -992,7 +1084,7 @@ test_texture3d(ustring filename, Mapping3D mapping)
 void
 test_texture3d_batch(ustring filename, Mapping3DWide mapping)
 {
-    Strutil::print("Testing 3d texture {}, output = {}\n", filename,
+    Strutil::print("Testing BATCHED texture3d {}, output = {}\n", filename,
                    output_filename);
     int nchannels = nchannels_override ? nchannels_override : 4;
     ImageSpec outspec(output_xres, output_yres, nchannels, TypeDesc::FLOAT);
@@ -1025,7 +1117,224 @@ static void test_shadow(ustring /*filename*/) {}
 
 
 
-static void test_environment(ustring /*filename*/) {}
+void
+env_region(ImageBuf& image, ustring filename, MappingEnv mapping,
+           ImageBuf* image_ds, ImageBuf* image_dt, ROI roi)
+{
+    TextureSystem::Perthread* perthread_info     = texsys->get_perthread_info();
+    TextureSystem::TextureHandle* texture_handle = texsys->get_texture_handle(
+        filename);
+    int nchannels = nchannels_override ? nchannels_override : image.nchannels();
+
+    TextureOpt opt;
+    initialize_opt(opt);
+
+    float* result    = OIIO_ALLOCA(float, nchannels);
+    float* dresultds = test_derivs ? OIIO_ALLOCA(float, nchannels) : NULL;
+    float* dresultdt = test_derivs ? OIIO_ALLOCA(float, nchannels) : NULL;
+    for (ImageBuf::Iterator<float> p(image, roi); !p.done(); ++p) {
+        Imath::V3f R, dRdx, dRdy;
+        mapping(p.x(), p.y(), R, dRdx, dRdy);
+        // if (p.x() == 0 && p.y() == 0)
+        //     Strutil::print("R = {}\n", R);
+        // Call the texture system to do the filtering.
+        bool ok = use_handle
+                      ? texsys->environment(texture_handle, perthread_info, opt,
+                                            R, dRdx, dRdy, nchannels, result,
+                                            dresultds, dresultdt)
+                      : texsys->environment(filename, opt, R, dRdx, dRdy,
+                                            nchannels, result, dresultds,
+                                            dresultdt);
+        if (!ok) {
+            std::string e = texsys->geterror();
+            if (!e.empty())
+                Strutil::print(std::cerr, "ERROR: {}\n", e);
+        }
+
+        // Save filtered pixels back to the image.
+        for (int i = 0; i < nchannels; ++i)
+            result[i] *= scalefactor;
+        image.setpixel(p.x(), p.y(), result);
+        if (image_ds)
+            image_ds->setpixel(p.x(), p.y(), dresultds);
+        if (image_dt)
+            image_dt->setpixel(p.x(), p.y(), dresultdt);
+    }
+}
+
+
+
+void
+env_region_batch(ImageBuf& image, ustring filename, MappingEnvWide mapping,
+                 ImageBuf* image_ds, ImageBuf* image_dt, ROI roi)
+{
+    using namespace Tex;
+    TextureSystem::Perthread* perthread_info     = texsys->get_perthread_info();
+    TextureSystem::TextureHandle* texture_handle = texsys->get_texture_handle(
+        filename);
+    int nchannels_img = image.nchannels();
+    int nchannels = nchannels_override ? nchannels_override : image.nchannels();
+
+    TextureOptBatch opt;
+    initialize_opt(opt);
+
+    FloatWide* result    = OIIO_ALLOCA(FloatWide, nchannels);
+    FloatWide* dresultds = test_derivs ? OIIO_ALLOCA(FloatWide, nchannels)
+                                       : nullptr;
+    FloatWide* dresultdt = test_derivs ? OIIO_ALLOCA(FloatWide, nchannels)
+                                       : nullptr;
+    for (int y = roi.ybegin; y < roi.yend; ++y) {
+        for (int x = roi.xbegin; x < roi.xend; x += BatchWidth) {
+            Imath::Vec3<FloatWide> R, dRdx, dRdy;
+            mapping(IntWide::Iota(x), y, R, dRdx, dRdy);
+            int npoints  = std::min(BatchWidth, roi.xend - x);
+            RunMask mask = RunMaskOn >> (BatchWidth - npoints);
+
+            // Call the texture system to do the filtering.
+            // if (y == 0 && x == 0)
+            //     Strutil::print("R = {}\n", R);
+            bool ok
+                = use_handle
+                      ? texsys->environment(texture_handle, perthread_info, opt,
+                                            mask, (const float*)&R,
+                                            (const float*)&dRdx,
+                                            (const float*)&dRdy, nchannels,
+                                            (float*)result, (float*)dresultds,
+                                            (float*)dresultdt)
+                      : texsys->environment(filename, opt, mask,
+                                            (const float*)&R,
+                                            (const float*)&dRdx,
+                                            (const float*)&dRdy, nchannels,
+                                            (float*)result, (float*)dresultds,
+                                            (float*)dresultdt);
+
+            if (!ok) {
+                std::string e = texsys->geterror();
+                if (!e.empty())
+                    Strutil::print(std::cerr, "ERROR: {}\n", e);
+            }
+
+            // Save filtered pixels back to the image.
+            for (int i = 0; i < nchannels; ++i)
+                result[i] *= scalefactor;
+            float* resultptr = (float*)image.pixeladdr(x, y);
+            // FIXME: simplify by using SIMD scatter
+            for (int i = 0; i < npoints; ++i)
+                for (int c = 0; c < nchannels; ++c)
+                    resultptr[c + i * nchannels_img] = result[c][i];
+            if (test_derivs) {
+                float* resultdsptr = (float*)image_ds->pixeladdr(x, y);
+                float* resultdtptr = (float*)image_dt->pixeladdr(x, y);
+                for (int c = 0; c < nchannels; ++c) {
+                    for (int i = 0; i < npoints; ++i) {
+                        resultdsptr[c + i * nchannels_img] = dresultds[c][i];
+                        resultdtptr[c + i * nchannels_img] = dresultdt[c][i];
+                    }
+                }
+            }
+        }
+    }
+}
+
+
+
+static void
+test_environment(ustring filename, MappingEnv mapping)
+{
+    Strutil::print("Testing environment {}, output = {}\n", filename,
+                   output_filename);
+    int nchannels = nchannels_override ? nchannels_override : 4;
+    ImageSpec outspec(output_xres, output_yres, nchannels, TypeDesc::FLOAT);
+    ImageBuf image(outspec);
+    TypeDesc fmt(dataformatname);
+    image.set_write_format(fmt);
+    OIIO::ImageBufAlgo::zero(image);
+    ImageBuf image_ds, image_dt;
+    if (test_derivs) {
+        image_ds.reset(outspec);
+        image_ds.set_write_format(fmt);
+        OIIO::ImageBufAlgo::zero(image_ds);
+        image_dt.reset(outspec);
+        image_dt.set_write_format(fmt);
+        OIIO::ImageBufAlgo::zero(image_dt);
+    }
+
+    for (int iter = 0; iter < iters; ++iter) {
+        // Trick: switch to second texture, if given, for second iteration
+        if (iter && filenames.size() > 1)
+            filename = filenames[1];
+        if (close_before_iter)
+            texsys->close_all();
+        ImageBufAlgo::parallel_image(
+            get_roi(image.spec()), nthreads, [&](ROI roi) {
+                env_region(image, filename, mapping,
+                           test_derivs ? &image_ds : nullptr,
+                           test_derivs ? &image_dt : nullptr, roi);
+            });
+    }
+
+    if (!image.write(output_filename))
+        Strutil::print(std::cerr, "Error writing {} : {}\n", output_filename,
+                       image.geterror());
+    if (test_derivs) {
+        if (!image_ds.write(output_filename + "-ds.exr"))
+            Strutil::print(std::cerr, "Error writing {} : {}\n",
+                           (output_filename + "-ds.exr"), image_ds.geterror());
+        if (!image_dt.write(output_filename + "-dt.exr"))
+            Strutil::print(std::cerr, "Error writing {} : {}\n",
+                           (output_filename + "-dt.exr"), image_dt.geterror());
+    }
+}
+
+
+
+static void
+test_environment_batch(ustring filename, MappingEnvWide mapping)
+{
+    Strutil::print("Testing BATCHED environment {}, output = {}\n", filename,
+                   output_filename);
+    int nchannels = nchannels_override ? nchannels_override : 4;
+    ImageSpec outspec(output_xres, output_yres, nchannels, TypeDesc::FLOAT);
+    ImageBuf image(outspec);
+    TypeDesc fmt(dataformatname);
+    image.set_write_format(fmt);
+    OIIO::ImageBufAlgo::zero(image);
+    ImageBuf image_ds, image_dt;
+    if (test_derivs) {
+        image_ds.reset(outspec);
+        image_ds.set_write_format(fmt);
+        OIIO::ImageBufAlgo::zero(image_ds);
+        image_dt.reset(outspec);
+        image_dt.set_write_format(fmt);
+        OIIO::ImageBufAlgo::zero(image_dt);
+    }
+
+    for (int iter = 0; iter < iters; ++iter) {
+        // Trick: switch to second texture, if given, for second iteration
+        if (iter && filenames.size() > 1)
+            filename = filenames[1];
+        if (close_before_iter)
+            texsys->close_all();
+        ImageBufAlgo::parallel_image(
+            get_roi(image.spec()), nthreads, [&](ROI roi) {
+                env_region_batch(image, filename, mapping,
+                                 test_derivs ? &image_ds : nullptr,
+                                 test_derivs ? &image_dt : nullptr, roi);
+            });
+    }
+
+    if (!image.write(output_filename))
+        Strutil::print(std::cerr, "Error writing {} : {}\n", output_filename,
+                       image.geterror());
+    if (test_derivs) {
+        if (!image_ds.write(output_filename + "-ds.exr"))
+            Strutil::print(std::cerr, "Error writing {} : {}\n",
+                           (output_filename + "-ds.exr"), image_ds.geterror());
+        if (!image_dt.write(output_filename + "-dt.exr"))
+            Strutil::print(std::cerr, "Error writing {} : {}\n",
+                           (output_filename + "-dt.exr"), image_dt.geterror());
+    }
+}
 
 
 
@@ -1075,6 +1384,7 @@ test_getimagespec_gettexels(ustring filename)
 
 
 
+#ifndef OIIO_CODE_COVERAGE
 static void
 test_hash()
 {
@@ -1109,7 +1419,7 @@ test_hash()
     for (int f = 0; f < nfiles; ++f) {
         for (int y = 0; y < res; y += tilesize) {
             for (int x = 0; x < res; x += tilesize, ++i) {
-                OIIO::pvt::TileID id(*icf[f], 0, 0, x, y, 0);
+                OIIO::pvt::TileID id(*icf[f], 0, 0, x, y, 0, 0, 1);
                 size_t h = id.hash();
                 hh += h;
             }
@@ -1127,7 +1437,7 @@ test_hash()
     for (int f = 0; f < nfiles; ++f) {
         for (int y = 0; y < res; y += tilesize) {
             for (int x = 0; x < res; x += tilesize, ++i) {
-                OIIO::pvt::TileID id(*icf[f], 0, 0, x, y, 0);
+                OIIO::pvt::TileID id(*icf[f], 0, 0, x, y, 0, 0, 1);
                 size_t h = id.hash();
                 ++fourbits[h & 0xf];
                 ++eightbits[h & 0xff];
@@ -1183,6 +1493,7 @@ test_hash()
 
     ImageCache::destroy(imagecache);
 }
+#endif
 
 
 
@@ -1511,7 +1822,7 @@ hashrand(int x, int y, int z, int c, int seed)
 static void
 make_temp_noise_file(string_view filename, int seed)
 {
-    ImageSpec spec(2048, 2048, 4,
+    ImageSpec spec(maketest_res, maketest_res, maketest_chans,
                    Filesystem::extension(filename) == ".exr" ? TypeHalf
                                                              : TypeUInt16);
     ImageBuf buf(spec);
@@ -1541,7 +1852,8 @@ static void
 make_test_files()
 {
     Timer timer;
-    int n = 0;
+    int n      = 0;
+    int pieces = udim_tests ? pieces_per_udim : 1;
     for (int i = 0; n < num_test_files; ++i) {
         std::string filename = Strutil::fmt::format(maketest_template, i);
         bool do_print
@@ -1551,29 +1863,22 @@ make_test_files()
             Strutil::print("Temp file {}: {}\n", i, filename);
             fflush(stdout);
         }
-        if (udim_tests) {
-            // Strutil::print("UDIM {}\n", filename);
-            for (int u = 0; u < pieces_per_udim && n < num_test_files; ++u) {
-                std::string udim_filename
-                    = Strutil::replace(filename, "<UDIM>",
-                                       Strutil::fmt::format("{:04d}", u + 1001));
-                if (do_print)
-                    Strutil::print("    {}\n", udim_filename);
-                if (!Filesystem::exists(udim_filename))
-                    make_temp_noise_file(udim_filename, i + u * 19);
-                ++n;
-            }
-            filenames.emplace_back(filename);
-        } else {
-            if (!Filesystem::exists(filename))
-                make_temp_noise_file(filename, i);
+        for (int u = 0; u < pieces && n < num_test_files; ++u) {
+            std::string fn
+                = Strutil::replace(filename, "<UDIM>",
+                                   Strutil::fmt::format("{:04d}", u + 1001));
+            if (udim_tests && do_print)
+                Strutil::print("    {}\n", fn);
+            if (!Filesystem::exists(fn))
+                make_temp_noise_file(fn, i + u * 19);
             ++n;
-            filenames.emplace_back(filename);
         }
+        filenames.emplace_back(filename);
     }
-    Strutil::print("Created {} test files in {}\n\n",
-                   filenames_to_delete.size(),
-                   Strutil::timeintervalformat(timer()));
+    if (runstats)
+        Strutil::print("Created {} test files in {}\n\n",
+                       filenames_to_delete.size(),
+                       Strutil::timeintervalformat(timer()));
     fflush(stdout);
 }
 
@@ -1670,9 +1975,11 @@ main(int argc, const char* argv[])
         iters = 0;
     }
 
+#ifndef OIIO_CODE_COVERAGE
     if (testhash) {
         test_hash();
     }
+#endif
 
     Imath::M33f scale;
     scale.scale(Imath::V2f(0.3, 0.3));
@@ -1729,7 +2036,8 @@ main(int argc, const char* argv[])
         Strutil::print("\n");
     } else if (iters > 0 && filenames.size()) {
         ustring filename(filenames[0]);
-        test_gettextureinfo(filenames[0]);
+        if (do_gettextureinfo)
+            test_gettextureinfo(filenames[0]);
         const char* texturetype = "Plain Texture";
         texsys->get_texture_info(filename, 0, ustring("texturetype"),
                                  TypeDesc::STRING, &texturetype);
@@ -1758,25 +2066,22 @@ main(int argc, const char* argv[])
         }
         if (!strcmp(texturetype, "Volume Texture")) {
             if (batch) {
-                if (nowarp)
-                    test_texture3d_batch(filename, map_default_3D);
-                else
-                    test_texture3d_batch(filename, map_warp_3D);
+                test_texture3d_batch(filename, map_default_3D);
             } else {
-                if (nowarp)
-                    test_texture3d(filename, map_default_3D);
-                else
-                    test_texture3d(filename, map_warp_3D);
+                test_texture3d(filename, map_default_3D);
             }
         }
         if (!strcmp(texturetype, "Shadow")) {
             test_shadow(filename);
         }
         if (!strcmp(texturetype, "Environment")) {
-            test_environment(filename);
+            if (batch) {
+                test_environment_batch(filename, map_env_latlong);
+            } else {
+                test_environment(filename, map_env_latlong);
+            }
         }
-        if (!udim_tests)
-            test_getimagespec_gettexels(filename);
+        test_getimagespec_gettexels(filename);
         if (runstats || verbose)
             Strutil::print("Time: {}\n", Strutil::timeintervalformat(timer()));
     }
