@@ -8,6 +8,9 @@ files = [ "psd_123.psd", "psd_123_nomaxcompat.psd", "psd_bitmap.psd",
 for f in files:
     command += info_command (OIIO_TESTSUITE_IMAGEDIR + "/" + f)
 
+# Test unassociated alpha metadata
+command += info_command (OIIO_TESTSUITE_IMAGEDIR + "/psd_123.psd", extraargs="--no-autopremult")
+
 command += info_command ("src/different-mask-size.psd")
 command += info_command ("src/layer-mask.psd")
 
