@@ -17,6 +17,7 @@ command += rw_command ("src", "separate.tif")
 # Test bugs we had until OIIO 2.4 for these corrupt file
 command += oiiotool ("--oiioattrib try_all_readers 0 --info -v src/corrupt1.tif", failureok = True)
 command += oiiotool ("--oiioattrib try_all_readers 0 --info -v src/crash-1633.tif", failureok = True)
-command += oiiotool ("--oiioattrib try_all_readers 0 --info -v src/crash-1643.tif", failureok = True)
+command += oiiotool ("--oiioattrib try_all_readers 0 --info src/crash-1643.tif -o out.exr", failureok = True)
+command += iconvert ("src/crash-1709.tif crash-1709.exr", failureok=True)
 
 outputs = [ "check1.tif", "out.txt" ]
